from scrapy.commands.version import Command as ScrapyCommand
from scrapy.utils.versions import scrapy_components_versions

import os_scrapy


class Command(ScrapyCommand):
    def short_desc(self):
        return "Print OS-Scrapy version"

    def run(self, args, opts):
        if opts.verbose:
            versions = scrapy_components_versions()
            versions.insert(0, ("OS-Scrapy", os_scrapy.__version__))
            width = max(len(n) for (n, _) in versions)
            patt = "%-{}s : %s".format(width)
            for name, version in versions:
                print(patt % (name, version))
        else:
            print("OS-Scrapy %s" % os_scrapy.__version__)



import inspect

from scrapy.commands.crawl import Command as ScrapyCommand
from scrapy.crawler import Crawler
from scrapy.exceptions import UsageError
from scrapy.utils.log import logger
from scrapy.utils.misc import load_object


def is_crawler_class(obj):
    return inspect.isclass(obj) and issubclass(obj, Crawler)


def load_crawler_class(class_path):
    obj = load_object(class_path)
    if is_crawler_class(obj):
        return obj


DEFAULT_CRAWLER_CLASS = f"{Crawler.__module__}.{Crawler.__name__}"


class Command(ScrapyCommand):
    default_settings = {
        "CRAWLER_CLASS": DEFAULT_CRAWLER_CLASS,
    }

    def add_options(self, parser):
        super(Command, self).add_options(parser)
        parser.add_option(
            "-c",
            "--crawler-class",
            metavar="CRAWLER_CLASS",
            help=f"set crawler class. default {DEFAULT_CRAWLER_CLASS}",
        )

    def process_options(self, args, opts):
        super(Command, self).process_options(args, opts)
        if opts.crawler_class:
            self.settings.set("CRAWLER_CLASS", opts.crawler_class, "cmdline")

    def _create_crawler(self, spname):
        c = self.settings.get("CRAWLER_CLASS")
        crawler_class = load_crawler_class(c)
        return crawler_class(
            self.crawler_process.spider_loader.load(spname), self.settings
        )

    def run(self, args, opts):
        if len(args) < 1:
            raise UsageError()
        elif len(args) > 1:
            raise UsageError(
                (
                    "running 'os-scrapy crawl' ",
                    "with more than one spider is no longer supported",
                )
            )
        spname = args[0]

        crawler = self._create_crawler(spname)
        logger.debug(f"Crawler {crawler.__module__}.{crawler.__class__.__name__}")

        crawl_defer = self.crawler_process.crawl(crawler, **opts.spargs)

        if getattr(crawl_defer, "result", None) is not None and issubclass(
            crawl_defer.result.type, Exception
        ):
            self.exitcode = 1
        else:
            self.crawler_process.start()

            if self.crawler_process.bootstrap_failed or (
                hasattr(self.crawler_process, "has_exception")
                and self.crawler_process.has_exception
            ):
                self.exitcode = 1



from os.path import abspath, exists, join

import os_scrapy_cookiecutter
from cookiecutter.main import cookiecutter
from scrapy.commands.startproject import Command as ScrapyCommand
from scrapy.exceptions import UsageError


class Command(ScrapyCommand):
    def add_options(self, parser):
        super(Command, self).add_options(parser)
        parser.add_option(
            "-p",
            "--package",
            dest="package",
            action="store_true",
            help="create project as package",
        )

    def run(self, args, opts):
        if not opts.package:
            return super(Command, self).run(args, opts)

        if len(args) not in (1, 2):
            raise UsageError()

        project_name = args[0]
        project_dir = args[0]

        if len(args) == 2:
            project_dir = args[1]

        if exists(join(project_dir, "scrapy.cfg")):
            self.exitcode = 1
            print("Error: scrapy.cfg already exists in %s" % abspath(project_dir))
            return

        if not self._is_valid_name(project_name):
            self.exitcode = 1
            return

        try:
            cookiecutter(
                os_scrapy_cookiecutter.TEMPLATE_DIR,
                no_input=True,
                extra_context={
                    "project_name": project_name,
                    "project_dir": project_dir,
                },
            )
        except Exception as e:
            self.exitcode = 1
            print(f"Error: create project with cookiecutter {e}")
            return



import cProfile
import optparse
import os
import sys

from scrapy.cmdline import (
    garbage_collect,
    _get_commands_from_entry_points,
    _get_commands_from_module,
    _pop_command_name,
    _run_print_help,
)
from scrapy.utils.project import get_project_settings, inside_project

import os_scrapy
from scrapy.crawler import CrawlerProcess
from os_scrapy.settings import default_settings


def _run_command_profiled(cmd, args, opts):
    if opts.profile:
        sys.stderr.write("os-scrapy: writing cProfile stats to %r\n" % opts.profile)
    loc = locals()
    p = cProfile.Profile()
    p.runctx("cmd.run(args, opts)", globals(), loc)
    if opts.profile:
        p.dump_stats(opts.profile)


def _run_command(cmd, args, opts):
    if opts.profile:
        _run_command_profiled(cmd, args, opts)
    else:
        cmd.run(args, opts)


def _print_header(settings, inproject):
    if inproject:
        print(
            "OS-Scrapy %s - project: %s\n"
            % (os_scrapy.__version__, settings["BOT_NAME"])
        )
    else:
        print("OS-Scrapy %s - no active project\n" % os_scrapy.__version__)


def _print_commands(settings, inproject):
    _print_header(settings, inproject)
    print("Usage:")
    print("  os-scrapy <command> [options] [args]\n")
    print("Available commands:")
    cmds = _get_commands_dict(settings, inproject)
    for cmdname, cmdclass in sorted(cmds.items()):
        print("  %-13s %s" % (cmdname, cmdclass.short_desc()))
    if not inproject:
        print()
        print("  [ more ]      More commands available when run from project directory")
    print()
    print('Use "os-scrapy <command> -h" to see more info about a command')


def _print_unknown_command(settings, cmdname, inproject):
    _print_header(settings, inproject)
    print("Unknown command: %s\n" % cmdname)
    print('Use "os-scrapy" to see available commands')


def _get_commands_dict(settings, inproject):
    cmds = _get_commands_from_module("scrapy.commands", inproject)
    cmds.update(_get_commands_from_module("os_scrapy.commands", inproject))
    cmds.update(_get_commands_from_entry_points(inproject))
    cmds_module = settings["COMMANDS_MODULE"]
    if cmds_module:
        cmds.update(_get_commands_from_module(cmds_module, inproject))
    return cmds


def _print_commands(settings, inproject):
    _print_header(settings, inproject)
    print("Usage:")
    print("  os-scrapy <command> [options] [args]\n")
    print("Available commands:")
    cmds = _get_commands_dict(settings, inproject)
    for cmdname, cmdclass in sorted(cmds.items()):
        print("  %-13s %s" % (cmdname, cmdclass.short_desc()))
    if not inproject:
        print()
        print("  [ more ]      More commands available when run from project directory")
    print()
    print('Use "os-scrapy <command> -h" to see more info about a command')


def execute(argv=None, settings=None):
    return _execute(argv, settings)


def _execute(argv=None, settings=None):
    if argv is None:
        argv = sys.argv

    if settings is None:
        settings = get_project_settings()
        settings.setmodule(default_settings, "default")
        # set EDITOR from environment if available
        try:
            editor = os.environ["EDITOR"]
        except KeyError:
            pass
        else:
            settings["EDITOR"] = editor

    inproject = inside_project()
    cmds = _get_commands_dict(settings, inproject)
    cmdname = _pop_command_name(argv)
    parser = optparse.OptionParser(
        formatter=optparse.TitledHelpFormatter(), conflict_handler="resolve"
    )
    if not cmdname:
        _print_commands(settings, inproject)
        sys.exit(0)
    elif cmdname not in cmds:
        _print_unknown_command(settings, cmdname, inproject)
        sys.exit(2)

    cmd = cmds[cmdname]
    parser.usage = "os-scrapy %s %s" % (cmdname, cmd.syntax())
    parser.description = cmd.long_desc()
    settings.setdict(cmd.default_settings, priority="command")
    cmd.settings = settings
    cmd.add_options(parser)
    opts, args = parser.parse_args(args=argv[1:])
    _run_print_help(parser, cmd.process_options, args, opts)

    cmd.crawler_process = CrawlerProcess(settings)
    _run_print_help(parser, _run_command, cmd, args, opts)
    sys.exit(cmd.exitcode)


if __name__ == "__main__":
    try:
        execute()
    finally:
        # Twisted prints errors in DebugInfo.__del__, but PyPy does not run gc.collect()
        # on exit: http://doc.pypy.org/en/latest/cpython_differences.html?highlight=gc.collect#differences-related-to-garbage-collection-strategies
        garbage_collect()



import pathlib

import os_scrapy


def test_version():
    version_file = (
        pathlib.Path(__file__).parents[1].joinpath("src/os_scrapy/VERSION").absolute()
    )
    assert os_scrapy.__version__ == open(version_file).read().strip()


if __name__ == "__main__":
    test_version()
